!(function ($) {
    /*!
     * SePlay DragNDrop
     *
     * Copyright 2018 SurveyEngine
     *
     * Depends:
     *  jquery, jquery.ui
     *
     * Example:
     * place this code snipped in an HTML before the matrix element that needs to be converted into a dragndrop
     *

     <link href="/lib/css/ui-lightness/jquery-ui.css" rel="stylesheet">
     <script src="/lib/jquery_ui/jquery-ui.js"></script>
     <script src="seplay.dragndrop.v1.1.js"></script>
     <script id="se_drag0">
     $(function(){
     var
     THIS_ID = "se_drag0",
     LABEL_SE_ITEM_DRAGNDROP_INSTRUCTION = "Drag the items from left to right in your own order",
     DEBUG = false;

     //  ------ settings end ----------------- //

     var
     dragndrop = new SePlay.DragNDrop({
     thisId: THIS_ID,
     instruction_label: LABEL_SE_ITEM_DRAGNDROP_INSTRUCTION,
     debug: DEBUG
     });

     })
     </script>

     *
     * Versionlog
     * 1.4 2019-05-09
     *  - fixed restoreState
     *  - improved mobile layout
     *  - improved height calculation
     *  - updated theme compatibility for dark themes
     *
     * 1.3.1
     *  - added example
     *
     * 1.3  2018-09-28
     *  - fixed missing-options-bug
     *  - added check for proper thisId reference
     *
     * 1.2  2018-09-19
     *  - improve container heights calculation
     *  - forced "hidden" for matrix
     *
     * 1.1  2018-08-17
     *  - added templates into script
     *
     * 1    2018-08-17
     *  - added touch support
     *  - finalized styles
     *  - added save / restore
     *  - added debug mode
     *
     */

    var LOG_LABEL = "dragndrop";

    var UTILS = {
        /**
         * waitFor
         * wait for 'condition' function to return true, then trigger success callback - after timeout trigger error callback
         * @param conditionFn
         * @param timeoutMs
         * @param succCb
         * @param errCb
         */
        waitFor: function (conditionFn, timeoutMs, succCb, errCb) {

            succCb = succCb || function () {
                };
            errCb = errCb || function () {
                };
            timeoutMs = timeoutMs !== undefined ? timeoutMs : 5000;

            var _return = $.Deferred();

            _return.done(succCb);
            _return.fail(errCb);

            var testRunner = setInterval(function () {
                    if (conditionFn()) {
                        clearAll();
                        _return.resolve();
                    }
                }, 250),
                testTimeout = setTimeout(function () {
                    clearAll();
                    _return.reject();
                }, timeoutMs),
                clearAll = function () {
                    clearInterval(testRunner);
                    clearTimeout(testTimeout);
                };

            return _return.promise();

        }
    };

    var TEMPLATES = {
        DRAGNDROP: $(' <div class="row se_drag"> <div class="col-xs-6"> <div>&nbsp;</div> <div class="se_drag_container se_drag_from" data-columnes="1 column"> <!-- div class="se_level_item se_drag_item"> <div>A</div> </div --> </div> <div>&nbsp;</div> </div> <div class="col-xs-6"> <div id="se_drag_label_first">First label</div> <div class="se_drag_container se_drag_to" data-columnes="1 column"> <div class="se_level_item se_drag_item hidden"></div> </div> <div id="se_drag_label_last">Last label</div> </div> </div>'),
        DRAGITEM: $('<div class="se_level_item se_drag_item" data-row-id><div></div></div>'),
        STYLES: $('' +
            '<style>' +
            '   .se_drag_hidden         { display: none !important; } ' +
            '   .se_drag_container      { min-height: 100px; background: rgba(255,255,255,.2); border: 1px solid rgba(102,102,102,.4); } ' +
            '   .se_drag_item, ' +
            '   .se_drag_placeholder    { min-height: 50px; padding: 1em; border: 1px solid rgba(102,102,102,.4); cursor: pointer;} ' +
            '   .se_drag_placeholder    { border: 1px dotted #f90; background: rgb(255, 201, 130); } ' +
            '   .se_drag_container .se_drag_item:nth-child(2n)      { background-color: rgba(0, 0, 0, .04); } ' +
            '   .se_drag_container .se_drag_item:nth-child(2n + 1)  { background-color: rgba(255,255,255,.2); } ' +
            '   .se_drag_item:hover     { background: #fd9 !important; } ' +
            '   .se_drag_item.ui-sortable-helper { background: #f90 !important; opacity: 0.7; } ' +
            '   @media screen and (max-width: 650px){ ' +
            '       .se_drag_item, .se_drag_placeholder{ padding: 0.5em; } ' +
            '   } ' +
            '</style> ')
    };

    var
        SeDragNDrop = function (params) {

            var v = {
                params: {
                    debug: false,
                    thisId: null,
                    instruction_label: "Drag the items from left to right in your own order",
                    matrix: "next"
                },
                elms: {
                    dragItems: [],
                    $dragndrop: null,
                    $drag_from: null,
                    $drag_to: null,
                    $matrix: null

                }
            };

            function readParams(params) {
                params = params || {};
                for (var i in params) {
                    v.params[i] = params[i];
                }
            }

            function createDragnDrop() {

                var
                    $this = v.params.thisId ? $('#' + v.params.thisId).closest('.se_item') : null,
                    $styles = TEMPLATES.STYLES.clone(),

                    $drag = TEMPLATES.DRAGNDROP.clone(),
                    $drag_from = $drag.find('.se_drag_from'),
                    $drag_to = $drag.find('.se_drag_to'),

                    $matrix, $matrixItm,
                    $labels, $options;

                if(!$this || !$this.length){
                    console.error(LOG_LABEL, "item not found for id", v.params.thisId, v.params);
                    return;
                }

                v.elms.$drag_from = $drag_from;
                v.elms.$drag_to = $drag_to;

                if (v.params.matrix !== "next") {
                    $matrix = $(v.params.matrix);
                    $matrixItm = $matrix.closest('.se_item');

                } else {
                    if(!$this){
                        console.error(LOG_LABEL, "item not found for id; cant use 'next' for matrix", v.params.thisId, v.params);
                        return;
                    }
                    $matrixItm = $this.next();
                }

                if(!$matrixItm.length){
                    console.error(LOG_LABEL, "matrix .se_item not found", v.params.matrix, v.params);
                    return;
                }

                $matrix = $matrixItm.find('.se_matrix');

                v.elms.$matrix = $matrix;

                if($this){
                    $this.hide();
                }



                if (v.params.debug !== true)
                    $matrix.addClass('se_drag_hidden');

                $labels = $matrixItm.find('.se_matrix_header > div');
                $drag.find('#se_drag_label_first').text($labels.first().text());
                $drag.find('#se_drag_label_last').text($labels.last().text());

                $matrixItm
                    .append($styles)
                    .append($drag);

                $matrixItm.find('.se_instruction').text(v.params.instruction_label);

                $options = $matrixItm.find('.se_matrix .row:not(:first-child) > div:first-child');

                if(!$options.length) return;

                var $dragItem, dragItems = [];
                $options.each(function (i, elm) {
                    $dragItem = TEMPLATES.DRAGITEM.clone();

                    $dragItem.attr('data-row-id', i);
                    $dragItem.find(" > div").text($(elm).text());

                    dragItems.push($dragItem);
                    $drag_from.append($dragItem);
                });

                v.elms.dragItems = dragItems;

                restoreState();

                setDragContainerHeight();


                v.elms.$dragndrop = $([$drag_from, $drag_to]).sortable({
                    se_drag_item: ".se_drag_item ",
                    forcePlaceholderSize: true,
                    connectWith: '.se_drag_container',
                    dropOnEmpty: true,
                    placeholder: "se_drag_placeholder",

                    stop: function (ev, ui) {
                        saveState(ev);
                    }
                });

            }


            function setDragContainerHeight(){
                var dragItems = v.elms.dragItems,
                    $dragItem = dragItems[0];

                UTILS.waitFor(function dragItemIsVisible() {
                    return $dragItem.is(':visible');
                }).then(function () {
                    setTimeout(function(){
                        updateDragContainerHeight();
                    }, 50);
                });

                var isResizing;
                $(window).on("resize", function(){
                    clearTimeout(isResizing);
                    isResizing = setTimeout(function(){
                        updateDragContainerHeight();
                    }, 100);
                });
            }

            function updateDragContainerHeight(){
                var dragItems = v.elms.dragItems,
                    $drag_from = v.elms.$drag_from,
                    $drag_to = v.elms.$drag_to;

                // this must be calculated because it is never for sure that the drag_from contains all
                // the options, e.g. on "prev" or after resize

                var borderWidth = parseInt($drag_from.css("border-width")),
                    dragItemsHeight = dragItems.reduce(function(r, $dragItem){ return r + $dragItem.outerHeight(); }, 0) + (2 * borderWidth);

                $drag_from.css('height', dragItemsHeight);
                $drag_to.css('height', dragItemsHeight);
            }

            function selectRow(rowElm, pos) {
                var $rowElm = $(rowElm),
                    input;

                console.debug("SAVE", rowElm, pos);
                // unselect input in row
                if (typeof pos === "undefined") {
                    if ($rowElm.find('.se_level_item.se_selected').length) {
                        $rowElm
                            .find('.se_level_item.se_selected')
                            .removeClass('se_selected')
                            .find('input')[0].checked = false;
                    }

                    return;
                }

                // select new input
                var $newPos = $rowElm.find('.se_level_item').eq(pos);
                if (!$newPos.hasClass('se_selected')) {
                    $newPos.addClass('se_selected');
                    $newPos.find('input')[0].checked = true;
                }

            }

            function restoreState() {
                var $rows = v.elms.$matrix.find('.row:not(:first-child)'),
                    moveRows = [];

                $rows.each(function (i, elm) {
                    var $row = $(elm),
                        pos = (+$row.find('.se_level_item.se_selected').find('input').val() || 0) - 1;

                    if (pos !== -1) {
                        moveRows[pos] = v.elms.$drag_from.find('.se_drag_item[data-row-id=' + i + ']');
                    }
                });

                $(moveRows).each(function (i, elm) {
                    $(elm).appendTo(v.elms.$drag_to);
                })
            }


            function saveState(ev) {
                console.debug("EV", ev);

                var $rows = v.elms.$matrix.find('.row:not(:first-child)');
                // de-select all
                $rows.each(function (i, elm) {
                    selectRow(elm)
                });

                // read in order of rows
                setTimeout(function () {
                    v.elms.$drag_to.find('.se_drag_item[data-row-id]').each(function (pos, elm) {
                        var rowId = +$(elm).attr('data-row-id'),
                            rowElm = $rows.eq(rowId);

                        // and save each pos to matrix
                        selectRow(rowElm, pos);
                    });
                }, 100)

            }

            this.destroy = function(){
                if(v.elms.$dragndrop)
                    v.elms.$dragndrop.sortable("destroy");
            };

            function validate() {}

            readParams(params);
            createDragnDrop();

            return this;
        },
        SePlay = window.SePlay = window.SePlay || {};

    SePlay.DragNDrop = SeDragNDrop;

}(jQuery));

/*!
 * jQuery UI Touch Punch 0.2.3
 *
 * Copyright 2011–2014, Dave Furfero
 * Dual licensed under the MIT or GPL Version 2 licenses.
 *
 * Depends:
 *  jquery.ui.widget.js
 *  jquery.ui.mouse.js
 */
!function (a) {
    function f(a, b) {
        if (!(a.originalEvent.touches.length > 1)) {
            a.preventDefault();
            var c = a.originalEvent.changedTouches[0], d = document.createEvent("MouseEvents");
            d.initMouseEvent(b, !0, !0, window, 1, c.screenX, c.screenY, c.clientX, c.clientY, !1, !1, !1, !1, 0, null), a.target.dispatchEvent(d)
        }
    }

    if (a.support.touch = "ontouchend" in document, a.support.touch) {
        var e, b = a.ui.mouse.prototype, c = b._mouseInit, d = b._mouseDestroy;
        b._touchStart = function (a) {
            var b = this;
            !e && b._mouseCapture(a.originalEvent.changedTouches[0]) && (e = !0, b._touchMoved = !1, f(a, "mouseover"), f(a, "mousemove"), f(a, "mousedown"))
        }, b._touchMove = function (a) {
            e && (this._touchMoved = !0, f(a, "mousemove"))
        }, b._touchEnd = function (a) {
            e && (f(a, "mouseup"), f(a, "mouseout"), this._touchMoved || f(a, "click"), e = !1)
        }, b._mouseInit = function () {
            var b = this;
            b.element.bind({
                touchstart: a.proxy(b, "_touchStart"),
                touchmove: a.proxy(b, "_touchMove"),
                touchend: a.proxy(b, "_touchEnd")
            }), c.call(b)
        }, b._mouseDestroy = function () {
            var b = this;
            b.element.unbind({
                touchstart: a.proxy(b, "_touchStart"),
                touchmove: a.proxy(b, "_touchMove"),
                touchend: a.proxy(b, "_touchEnd")
            }), d.call(b)
        }
    }
}(jQuery);